"""
iacli library
~~~~~~~~~~~~~~~~~~~~~~~

Iacli is the CLI driver for the internetarchive python library:

<https://github.com/jjjake/ia-wrapper>.

:copyright: (c) 2013 by Jacob M. Johnson.
:license: GPL, see LICENSE for more details.

"""

from internetarchive import __version__



__title__ = 'iacli'
__version__ = __version__
__author__ = 'Jacob M. Johnson'
__license__ = 'GPL'
__copyright__ = 'Copyright 2013 Jacob M. Johnson'
